module FriendlyScore
  class API
    attr_reader :application_id, :application_secret

    # A new instance of API.
    #
    # @param application_id [Integer] application id
    # @param application_secret [String] application secret
    def initialize application_id, application_secret
      @application_id = application_id
      @application_secret = application_secret
    end

    # Returns user data (api: 'api/show')
    #
    # @param id [Integer] identificator of user
    # @return [FriendlyScore::User]
    def show id
      endpoint = get_endpoint Endpoints::Show
      endpoint.perform id
    end

    # Returns partner user
    #
    # @param partner_user_id [String] identificaotr of partner user
    # @return [FriendlyScore::User] 
    def show_partner_user partner_user_id
      endpoint = get_endpoint Endpoints::ShowPartnerUser
      endpoint.perform partner_user_id
    end

    # Sets is user positive or not
    #
    # @param id [Integer] users identificator
    # @param positive [Integer] 1 if user is positive, 0 if not
    def user_positive id, positive
      endpoint = get_endpoint Endpoints::UserPositive
      endpoint.validate id, positive
      endpoint.perform id, positive
    end

    # Sets users status with description
    #
    # @param id [Integer] users identificator
    # @param status [Integer] any integer number
    # @param status_description [String] status description
    def user_status id, status, status_description
      endpoint = get_endpoint Endpoints::UserStatus
      endpoint.perform id, status, status_description
    end

    # List of all application users
    #
    # @return [List<FriendlyScore::User>] list of all users
    def users
      endpoint = get_endpoint Endpoints::Users
      endpoint.perform
    end

    private

    def get_endpoint endpoint_class
      @endpoints ||= {}
      unless @endpoints[endpoint_class]
        @endpoints[endpoint_class] = endpoint_class.new(
          @application_id, @application_secret, responder
        )
      end
      @endpoints[endpoint_class]
    end

    def responder
      @responder ||= Responder.new
    end
  end
end
