module FriendlyScore
  module Endpoints
    class Base 
      def initialize application_id, application_secret, responder
        @application_id = application_id
        @application_secret = application_secret
        @responder = responder
      end

      def perform *args
        code, body = make_request args
        case code
          when "200" then process_response body
          when "404" then process_error body
          else raise Errors::ServerError
        end
      end

      private

      def make_request args
        params = authorized_request_params args
        @responder.get api_path, params 
      end

      def authorized_request_params args
        {
          "appId" => @application_id,
          "appSecret" => @application_secret
        }.merge request_params *args
      end

      def process_error body
        message = MultiJson.load(body)["message"]
        raise Errors::NotFoundError.new message
      end
    end
  end
end
